import re
import os
import requests
import grequests
from QuranDownloader.verse_key_converters import verse_key_to_num
from QuranDownloader.concurrent_req import *


def word_spreader(label):
    # remove tajweed hokm in between words
    label = re.sub('<tajweed class=\w+>(\w* \w*)<\/tajweed>', '\g<1>', label)
    # split at any word space
    return re.split('(?<!tajweed) ', label)

# download word audio file


url = 'https://verses.quran.com/wbw/{}.mp3'


def download_word(verses_keys, export_dir='./export/words'):
    url_list = []
    for verse_key in verses_keys:
        quran_req = concurrent_req(
            requests.get, f'https://api.quran.com/api/v4/quran/verses/imlaei?verse_key={verse_key}').json()
        # try to get the verse
        try:
            quran_data = quran_req['verses'][0]['text_imlaei']
        except:
            return print('{verse_key} is not a valid verse key')
        # loop throw all word to fill the requests
        for i, word in enumerate(quran_data.split(' ')):
            word_key = f"{verse_key}:{i + 1}"
            link = url.format(verse_key_to_num(word_key))
            url_list.append(grequests.get(link))
    # create export dir if not exists
    if not os.path.exists(export_dir):
        os.makedirs(export_dir)
    # do requests
    for file_res in concurrent_req(grequests.map, url_list):
        name = file_res.url.split('/')[-1]
        open(f"{export_dir}/{name}", "wb").write(file_res.content)



from QuranDownloader.verse_key_converters import *
from QuranDownloader.consts import *
from QuranDownloader.download_word import download_word
from QuranDownloader.download_verse import download_verse


# download_word(verse_key='1:5', export_dir='./export/words')


RECITERS = ['AbdulBaset_Mujawwad', 'AbdulBaset_Murattal', 'Sudais', 'Shatri', 'Rifai', 'Husary',
            'Alafasy', 'Minshawi_Mujawwad', 'Minshawi_Murattal', 'Shuraym', 'Mohammad_al_Tablaway', 'Husary_Muallim']
SHIKS_LIMITS = [1, 13]
FILTERS = ['chapter_number', 'juz_number', 'page_number', 'verse_key']
API_URL = 'https://verses.quran.com'
RECITATIONS_URL = 'https://api.quran.com/api/v4/quran/recitations'


import concurrent.futures
# make concurrent requests


def concurrent_req(fun, url):
    executor = concurrent.futures.ProcessPoolExecutor()
    future = executor.submit(fun, url)
    return future.result()



import os
import re
import requests
import grequests
from QuranDownloader.consts import *
from QuranDownloader.concurrent_req import *


url_list = []
# get info urls


def info_and_download(req_url, export_dir):
    # do info request
    info_res = concurrent_req(requests.get, req_url).json()
    print(f"reciter_name: {info_res['meta']['reciter_name']}")
    print(req_url)
    # download
    if (len(info_res['audio_files']) < 1):
        raise Exception('invalid parameters')
    for res in info_res['audio_files']:
        audio_url = res['url'].replace('//', 'https://') if (
            'mirrors' in res['url']) else f"{API_URL}/{res['url']}"
        # get audio file request
        url_list.append(concurrent_req(grequests.get, audio_url))

# download verses


def download_verse(
    reciter_name,
    verse_key=None,
    export_dir='./export',
    chapter_number=None,
    juz_number=None,
    page_number=None,
):
    # eval reciter id
    if reciter_name != 'all':
        # try:
        reciter_id = RECITERS.index(reciter_name) + 1
        if not reciter_id in range(SHIKS_LIMITS[0], SHIKS_LIMITS[1]):
            raise Exception('{reciter_name} is not a valid reciter name\ncheck the all list from "RECITERS"')
        # except:
        #     raise Exception(f'{reciter_name} is not a valid reciter name\ncheck the all list from "RECITERS"')
    #
    queries = ''
    # prepare url filters
    for filter in FILTERS:
        filter_value = eval(filter)
        if (filter_value):
            queries += '?' if queries.find('?') == -1 else '&'
            queries += f'{filter}={filter_value}'
    # create export dir if not exists
    if not os.path.exists(export_dir):
        os.makedirs(export_dir)
    # do requests
    try:
        if reciter_id == 'all':
            for i in range(SHIKS_LIMITS[0], SHIKS_LIMITS[1]):
                req_url = f'{RECITATIONS_URL}/{i}{queries}'
                info_and_download(req_url, f'{export_dir}/{i}')
        else:
            req_url = f'{RECITATIONS_URL}/{reciter_id}{queries}'
            info_and_download(req_url, export_dir)
    except Exception as err:
        print(f'{err} {queries}')
    # send requests
    for file_res in concurrent_req(grequests.map, url_list):
        name = re.findall(r'\d+.mp3$', file_res.url)[0]
        # create export dir if not exists
        folder_name = re.findall(
            r'(\.com(/everyayah)?/(.+)/(?=(mp3/)?\d+.mp3))', file_res.url)
        folder_name = folder_name[0][2].replace('/', '_')
        folder_name = f'{export_dir}/{folder_name}'
        if not os.path.exists(folder_name):
            os.makedirs(folder_name)
        # write the audio file
        open(f"{folder_name}/{name}", "wb").write(file_res.content)
